@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -8,14 +8,20 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations # isort:skip
+
 import pytest ; pytest
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
+# Standard library imports
+from typing import Dict
+
 # Bokeh imports
 from bokeh._testing.util.filesystem import with_file_contents
+from bokeh.application.handlers.handler import Handler
 from bokeh.document import Document
 
 # Module under test
@@ -44,19 +50,19 @@ def on_session_destroyed(session_context):
 # Dev API
 #-----------------------------------------------------------------------------
 
-class Test_ServerLifecycleHandler(object):
 
+class Test_ServerLifecycleHandler:
     # Public methods ----------------------------------------------------------
 
     async def test_empty_lifecycle(self) -> None:
         doc = Document()
-        out = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = bahs.ServerLifecycleHandler(filename=filename)
             handler.modify_document(doc)
-            out['handler'] = handler
+            result['handler'] = handler
         with_file_contents("# This script does nothing", load)
-        handler = out['handler']
+        handler = result['handler']
         handler.on_server_loaded(None)
         handler.on_server_unloaded(None)
         await handler.on_session_created(None)
@@ -66,8 +72,8 @@ class Test_ServerLifecycleHandler(object):
         assert not doc.roots
 
     def test_lifecycle_bad_syntax(self) -> None:
-        result = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = bahs.ServerLifecycleHandler(filename=filename)
             result['handler'] = handler
         with_file_contents("This is a syntax error", load)
@@ -77,8 +83,8 @@ class Test_ServerLifecycleHandler(object):
         assert 'Invalid syntax' in handler.error
 
     def test_lifecycle_runtime_error(self) -> None:
-        result = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = bahs.ServerLifecycleHandler(filename=filename)
             result['handler'] = handler
         with_file_contents("raise RuntimeError('nope')", load)
@@ -88,8 +94,8 @@ class Test_ServerLifecycleHandler(object):
         assert 'nope' in handler.error
 
     def test_lifecycle_bad_server_loaded_signature(self) -> None:
-        result = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = bahs.ServerLifecycleHandler(filename=filename)
             result['handler'] = handler
         with_file_contents("""
@@ -99,13 +105,14 @@ def on_server_loaded(a,b):
 
         handler = result['handler']
         assert handler.error is not None
+        assert handler.error_detail is not None
         assert 'on_server_loaded must have signature func(server_context)' in handler.error
         assert 'func(a, b)' in handler.error
         assert "Traceback" in handler.error_detail
 
     def test_lifecycle_bad_server_unloaded_signature(self) -> None:
-        result = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = bahs.ServerLifecycleHandler(filename=filename)
             result['handler'] = handler
         with_file_contents("""
@@ -115,13 +122,14 @@ def on_server_unloaded(a,b):
 
         handler = result['handler']
         assert handler.error is not None
+        assert handler.error_detail is not None
         assert 'on_server_unloaded must have signature func(server_context)' in handler.error
         assert 'func(a, b)' in handler.error
         assert "Traceback" in handler.error_detail
 
     def test_lifecycle_bad_session_created_signature(self) -> None:
-        result = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = bahs.ServerLifecycleHandler(filename=filename)
             result['handler'] = handler
         with_file_contents("""
@@ -135,8 +143,8 @@ def on_session_created(a,b):
         assert 'func(a, b)' in handler.error
 
     def test_lifecycle_bad_session_destroyed_signature(self) -> None:
-        result = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = bahs.ServerLifecycleHandler(filename=filename)
             result['handler'] = handler
         with_file_contents("""
@@ -150,8 +158,8 @@ def on_session_destroyed(a,b):
         assert 'func(a, b)' in handler.error
 
     async def test_calling_lifecycle_hooks(self) -> None:
-        result = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = result['handler'] = bahs.ServerLifecycleHandler(filename=filename)
             if handler.failed:
                 raise RuntimeError(handler.error)
@@ -163,13 +171,9 @@ def on_session_destroyed(a,b):
         assert "on_session_created" == await handler.on_session_created(None)
         assert "on_session_destroyed" == await handler.on_session_destroyed(None)
 
-    def test_missing_filename_raises(self) -> None:
-        with pytest.raises(ValueError):
-            bahs.ServerLifecycleHandler()
-
     def test_url_path(self) -> None:
-        result = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = bahs.ServerLifecycleHandler(filename=filename)
             result['handler'] = handler
         with_file_contents("""
@@ -179,11 +183,12 @@ def on_server_unloaded(server_context):
 
         handler = result['handler']
         assert handler.error is None
-        assert handler.url_path().startswith("/")
+        url_path = handler.url_path()
+        assert url_path is not None and url_path.startswith("/")
 
     def test_url_path_failed(self) -> None:
-        result = {}
-        def load(filename):
+        result: Dict[str, Handler] = {}
+        def load(filename: str):
             handler = bahs.ServerLifecycleHandler(filename=filename)
             result['handler'] = handler
         with_file_contents("""
